@@ -29,11 +29,9 @@ struct apk_provider;
 #define APK_SCRIPT_TRIGGER		6
 #define APK_SCRIPT_MAX			7
 
-#define APK_SIGN_NONE			0
 #define APK_SIGN_VERIFY			1
 #define APK_SIGN_VERIFY_IDENTITY	2
-#define APK_SIGN_GENERATE		4
-#define APK_SIGN_VERIFY_AND_GENERATE	5
+#define APK_SIGN_VERIFY_AND_GENERATE	3
 
 #define APK_DEP_IRRELEVANT		0x01
 #define APK_DEP_SATISFIES		0x02
@@ -49,14 +47,17 @@ struct apk_sign_ctx {
 	int action;
 	const EVP_MD *md;
 	int num_signatures;
-	int control_started : 1;
-	int data_started : 1;
-	int has_data_checksum : 1;
-	int control_verified : 1;
-	int data_verified : 1;
+	int verify_error;
+	unsigned char control_started : 1;
+	unsigned char data_started : 1;
+	unsigned char has_data_checksum : 1;
+	unsigned char control_verified : 1;
+	unsigned char data_verified : 1;
+	unsigned char end_seen : 1;
 	char data_checksum[EVP_MAX_MD_SIZE];
 	struct apk_checksum identity;
 	EVP_MD_CTX *mdctx;
+	EVP_MD_CTX *idctx;
 
 	struct {
 		apk_blob_t data;
@@ -96,28 +97,29 @@ struct apk_installed_package {
 
 struct apk_package {
 	apk_hash_node hash_node;
-	unsigned int foreach_genid;
+	struct apk_name *name;
+	struct apk_installed_package *ipkg;
+	struct apk_dependency_array *depends, *install_if, *provides;
+	apk_blob_t *version;
+	size_t installed_size, size;
+
 	union {
 		struct apk_solver_package_state ss;
 		int state_int;
-		void *state_ptr;
 	};
-	struct apk_name *name;
-	struct apk_installed_package *ipkg;
-	apk_blob_t *version, *arch, *license;
-	apk_blob_t *origin, *maintainer;
-	char *url, *description, *commit;
-	char *filename;
-	struct apk_dependency_array *depends, *install_if, *provides;
-	size_t installed_size, size;
-	time_t build_time;
+	unsigned int foreach_genid;
 	unsigned short provider_priority;
-	unsigned repos : APK_MAX_REPOS;
-	unsigned seen : 1;
-	unsigned marked : 1;
-	unsigned uninstallable : 1;
-	unsigned cached_non_repository : 1;
+	unsigned short repos;
+	unsigned short filename_ndx;
+	unsigned char seen : 1;
+	unsigned char marked : 1;
+	unsigned char uninstallable : 1;
+	unsigned char cached_non_repository : 1;
 	struct apk_checksum csum;
+
+	time_t build_time;
+	apk_blob_t *arch, *license, *origin, *maintainer;
+	char *url, *description, *commit;
 };
 APK_ARRAY(apk_package_array, struct apk_package *);
 
@@ -125,7 +127,8 @@ APK_ARRAY(apk_package_array, struct apk_package *);
 #define APK_PROVIDER_FROM_PROVIDES(pkg,p) (struct apk_provider){(pkg),(p)->version}
 
 #define PKG_VER_FMT		"%s-" BLOB_FMT
-#define PKG_VER_PRINTF(pkg)	pkg->name->name, BLOB_PRINTF(*pkg->version)
+#define PKG_VER_PRINTF(pkg)	(pkg)->name->name, BLOB_PRINTF(*(pkg)->version)
+#define PKG_VER_STRLEN(pkg)	(strlen(pkg->name->name) + 1 + pkg->version->len)
 #define PKG_FILE_FMT		PKG_VER_FMT ".apk"
 #define PKG_FILE_PRINTF(pkg)	PKG_VER_PRINTF(pkg)
 
@@ -134,6 +137,7 @@ extern const char *apk_script_types[];
 void apk_sign_ctx_init(struct apk_sign_ctx *ctx, int action,
 		       struct apk_checksum *identity, int keys_fd);
 void apk_sign_ctx_free(struct apk_sign_ctx *ctx);
+int apk_sign_ctx_status(struct apk_sign_ctx *ctx, int tar_rc);
 int apk_sign_ctx_process_file(struct apk_sign_ctx *ctx,
 			      const struct apk_file_info *fi,
 			      struct apk_istream *is);
